<?php
namespace FiloBlu\Flow\Model\From;

class Eavs extends AbstractFrom {

    const PRODUCT_NOT_FOUND_ERR_CODE = 1;
    const SKU_ATTRIBUTE_ERR_CODE = 2;
    const DROPDOWN_STORE_ERR_CODE = 3;
    const ENTITY_MANAGER_CLASS_PREFIX = 'FiloBlu\Flow\Model\From\Entity\\';
    const DEFAULT_ENTITY_TYPE_CODE = 'catalog_product';
    const ATTRIBUTE_ENTITY_TYPE_SUFFIX = 'attribute';
    const PRODUCT_ATTRIBUTE_ENTITY_TYPE_CODE = 'product_attribute';
    const EMAIL_FROM = "Eavs processor <magenteam@.it>";
    const EMAIL_TO = "";
    const PRODUCT_REF_ID = 0;
    const ATTRIBUTE_REF_ID = 1;
    const SPECIAL_REF_ID = 2;
    const CONFIG_FLOW_BULKMODE = 'filoblu_flow/flow_process/flow_update_bulkmode';
    const CONFIG_FLOW_BULKTHRESHOLD = 'filoblu_flow/flow_process/flow_update_bulkmodethreshold';
    const EXECUTED_ON_ATTRIBUTE = 'attribute';
    const EXECUTED_ON_PRODUCT = 'product';
    
    protected $_dontRetryErrorCodes = true;

    protected $_productEntityTypeId = 4;
    protected $_excludeFromBulk = array('sku');
    protected $_bulkModel = null;
    protected $_scopeConfig = null;


    /**
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource $resource
     * @param \Magento\Framework\Data\Collection\Db $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \FiloBlu\Flow\Helper\Data $helper_flow,
        \FiloBlu\Flow\Model\From\Entity\Bulk $bulkModel,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $this->helper_flow = $helper_flow;
        $this->_bulkModel = $bulkModel;
        $this->_scopeConfig = $scopeConfig;
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    
    /**
     * @return void
     */
    public function _construct()
    {
        $this->_init('FiloBlu\Flow\Model\ResourceModel\From\Eavs', 'meta_id');
    }

    protected function preprocessFileRows($query,$file) {


        $bulk_enabled = $this->_scopeConfig->getValue(self::CONFIG_FLOW_BULKMODE);

        if (!$bulk_enabled)
            return $query;

        $attributesToCopy = $this->_registry->registry('attribute_configurable_copy');
        $startTime = time();
        try {
            $this->_bulkModel->_insertMissingAttributeOptionValues($file);
            $this->_logger->info("bulk process : missing attribute options inserted in " . (time() - $startTime) . "s");
        } catch (\Exception $e){
            $this->_logger->info($e->getMessage());
        }


        $startTime = time();
        try {
            $this->_bulkModel->_setMetaStatusToNotFoundProducts($file);
            $this->_logger->info("bulk process : meta status for not found products in " . (time() - $startTime) . "s");
        } catch (\Exception $e){
            $this->_logger->info($e->getMessage());
        }

        $startTime = time();

        $loop = 0;
        $limit = is_numeric($this->_scopeConfig->getValue(self::CONFIG_FLOW_BULKTHRESHOLD)) ? $this->_scopeConfig->getValue(self::CONFIG_FLOW_BULKTHRESHOLD) : 5000;
        $count = 0;
        $offset = 0;

        while ($rows = $this->_bulkModel->_getMetaAttributesforProducts($file,$limit,$offset)) {

            foreach ($rows as $backend_model => $items) {
                $count = $count + (int)$this->_dumpEAVS($items);
            }

            $loop++;
            $offset = ($limit * $loop)- $count;


        }

        $this->_logger->info("bulk process : values to update got in " . (time() - $startTime) . "s");

        return $query;

    }

    protected function _dumpEAVS($items) {

        if (empty($items))
            return false;

        $targetTable = $items[0]['backend_type'];

        //Mage::log("Doing " . sizeof($items) . " of " . $items[0]['backend_type'], 0, "eavs.log");
        $this->_logger->info("Doing " . sizeof($items) . " of " . $items[0]['backend_type']);

        return $this->_saveAttributes($items, $targetTable);
    }

    protected function _saveAttributes($items, $targetTable) {
        if (empty($items))
            return false;

        $Connection = $this->getResource()->getConnection();

        $metaIds = array();
        $toUpdate = array();
        for ($i = 0; $i < sizeof($items); $i++) {
            $toUpdate[] = sprintf("(%s, %s, %s, %s)", $items[$i]['attribute_id'], $items[$i]['store_id'], $items[$i][$this->getEntityField()], $Connection->quote($items[$i]['v']));
            $metaIds[] = $items[$i]['meta_id'];
        }

        $updateQuery = "INSERT INTO `catalog_product_entity_{$targetTable}`
                            (`attribute_id`, `store_id`, `{$this->getEntityField()}`, `value`)
                        VALUES ";
        $updateQuery .= implode(', ', $toUpdate);
        $updateQuery .= " ON DUPLICATE KEY UPDATE `value` = VALUES(`value`);";

        $metaUpdateQuery = "UPDATE `flow_from_eavs` SET `meta_processed` = 1 WHERE `meta_id` IN (" . implode(', ', $metaIds) . ");";

        try {
            $Connection->beginTransaction();
            $Connection->query($updateQuery);
            $Connection->query($metaUpdateQuery);
            $Connection->commit();
            return sizeof($items);
        } catch (Exception $e) {
            Mage::log(__METHOD__ . ':' . __LINE__ . ': Rollback happened.' . $e->getMessage());
            $Connection->rollback();
            return false;
        }
    }
    
    public function process() {

        // $this->_logger->info(__METHOD__);

        /* check store view */
        $storesId = $this->getStoreId();
        if ($storesId === false) {
            return $this->error(false, "Store view " . $this->getS() . " is missing");
        }

        $entityTypeCode = $this->getMetaRefId();
        $executedOn = trim($this->getExecutedOn());
        $messagee = "Unknown special action type of execution or unknown EAVS action type (e: {$this->getE()} | a: {$this->getA()} | a: {$this->getV()} | meta_ref_id: {$entityTypeCode} | executed_on: {$executedOn})";

        // Default EAVS action, on product!
        if($entityTypeCode == 0 && strlen($executedOn) < 1){
            $entityTypeCode = self::DEFAULT_ENTITY_TYPE_CODE;
        // Special action on product
        } elseif($entityTypeCode > 0 && $executedOn == self::EXECUTED_ON_PRODUCT) {
            $entityTypeCode = self::DEFAULT_ENTITY_TYPE_CODE;
        // Special action on attribute
        } elseif($entityTypeCode > 0 && $executedOn == self::EXECUTED_ON_ATTRIBUTE){
            $entityTypeCode = self::PRODUCT_ATTRIBUTE_ENTITY_TYPE_CODE;
        } else {
            $this->_logger->info($messagee);
            return $this->error(false, $messagee);
        }

        $entityManagerClassName = $this->getEntityManagerClassName($entityTypeCode);
        try {
            foreach($storesId as $storeId) {
                $entityManager = \Magento\Framework\App\ObjectManager::getInstance()->create($entityManagerClassName,
                ['data' => [
                    'entityIdentifier' => $this->getE(),
                    'attributeCode' =>  $this->getA(),
                    'attributeValue' => stripcslashes($this->getV()),
                    'entityTypeCode' => $entityTypeCode,
                    'storeId' => $storeId
                    ]
                ]);
                $entityManager->process();
            }
        } catch (\Exception $e) {
            $this->_logger->info($e->getMessage());
            //if(array_key_exists($e->getCode(),$this->_dontRetryErrorCodes)){
                return $this->error(false, $e->getMessage());
            //}
            return $this->error(true, $e->getMessage());
        }

        return $this;
    }
    

    protected function getEntityManagerClassName($entityTypeCode) {
        if ($entityTypeCode == self::DEFAULT_ENTITY_TYPE_CODE) {
            return self::ENTITY_MANAGER_CLASS_PREFIX . 'Product';
        } elseif (strstr($entityTypeCode, self::ATTRIBUTE_ENTITY_TYPE_SUFFIX) !== false) {
            return self::ENTITY_MANAGER_CLASS_PREFIX . 'Attribute';
        }

        return self::ENTITY_MANAGER_CLASS_PREFIX . 'AbstractEntity';
    }

    protected function getStoreId() {
        $storeView = $this->getS();
        $language = $this->getL();
        $allStore = $this->helper_flow->getAllStoreIds();
        /* storeview is in s tag with codename */
        if (!empty($storeView)) {
            if(strtolower(trim($storeView)) == 'default'){
                $stores[] = 0;
                return $stores;
            }
            if (isset($allStore[$storeView])) {
                $stores[] = $allStore[$storeView];
                return $stores;
            }
        }
        elseif(!empty($language)){
            $languages = $this->helper_flow->getListLanguageForAllStores();
            if(isset($languages[strtolower($language)])){
                return $languages[strtolower($language)];
            }
        } else {
            $stores[] = 0;
            return $stores;//Mage_Core_Model_App::ADMIN_STORE_ID; //usually 0
        }

        return false;
    }

    protected function getEntityField() {

        return  'row_id';
        //$edition = $this->_productMetadata->getEdition();
        return ($edition == 'Enterprise' || $edition == 'B2B') ? 'row_id' : 'entity_id';

    }

    
}